# This file contains standard IO stuff

readGeneFamilies<-function(file, type.family="detect", sep="\t"){

  data<-read.table(file, header=TRUE, comment="", quote="", sep=sep)
  cnames<-colnames(data)
  ncols<-dim(data)[2]
  if(cnames[1]!="GeneFamily"){
    stop(sprintf("Error reading %s: Column one needs to be named \"GeneFamily\"\n", file))
  }
    
  if(tolower(type.family)=="detect"){
    FamilyType<-detectGeneFamily(data[,"GeneFamily"])
    if(is.null(FamilyType)){
      cat(sprintf("Could not detect gene family type. Using default (COG).\n"))
      FamilyType<-"COG"
    } else {
      cat(sprintf("Gene family type %s detected.\n", FamilyType))
    }
  } else {
    FamilyType<-type.family
  }

  res.familycheck<-checkFamilyType(data[, "GeneFamily"], type=FamilyType)
  
  if(res.familycheck==FALSE){
    stop("Failed to validate all entries in data file.\n")
  }

  DataStruct<-list()
  DataStruct$Data<-data[,2:ncols]
  DataStruct$Type<-FamilyType
  DataStruct$Annotation<-data.frame(as.character(data[,1]), getAnnotation(as.character(data[,1]), DataStruct$Type))
  colnames(DataStruct$Annotation)<-c("GeneFamily", "Annotation")
  
  return(DataStruct)
}

detectGeneFamily<-function(gene.families){

  Patterns<-getSupportedFamilies()
  nPatterns<-length(Patterns)
  
  for(i in 1:nPatterns){

    res.match<-grep(Patterns[[i]], gene.families)

    if(length(res.match)>=length(gene.families)/2){
      return(names(Patterns)[i])
    }

  }
  
  return(NULL)  
}

checkFamilyType<-function(gene.families, type){

  Patterns<-getSupportedFamilies()
  
  if(is.null(Patterns[[type]])){
    stop(sprintf("Unknown gene family type: %s.\n", type))
  }
  
  res.match<-grep(Patterns[[type]], gene.families)
  
  diff<-setdiff(1:length(gene.families), res.match)
  if(length(diff)>0){
    stop(sprintf("Error! The following rows could not be found in the gene family: %s.", paste(diff, collapse=",")))
  }
  
  return(TRUE)
				       
}

getAnnotation<-function(gene.families, type){

  if(missing(type)){
    stop("Gene family type is not specified.")
  }

  # Check for available annotation data
  if(!is.null(ShotgunFunctionalizeRAnnotationData$DESCRIPTIONS[[type]])){

    # Temporary data struct
    data.tmp<-data.frame(gene.families)
    colnames(data.tmp)<-as.character(type)
    
    # Merge with annotation data    
    Annotation<-merge(x=data.tmp, y=ShotgunFunctionalizeRAnnotationData$DESCRIPTIONS[[type]], by.x=type, by.y=type, all.x=TRUE, all.y=FALSE, sort=FALSE)
    
    ReOrder<-match(as.character(data.tmp[,1]), as.character(Annotation[,1]))
    Annotation<-Annotation[ReOrder,]

    return(as.character(Annotation[,2]))
    
  } else {
  
    # Return NA if no annotation is available
    return(rep(NA, length(gene.families)))
    
  }
}

getSupportedFamilies<-function(){
  Patterns<-list(COG="COG[0-9]", EC="EC[0-9]", ARGENE="ARGENE[0-9]")
  return(Patterns)
}

writeRankings<-function(Result, File, col.names=TRUE, sep="\t"){
  write.table(Result$Ranking, file=file, col.names=col.names, row.names=FALSE, quote=FALSE, sep=sep)
}

removeZeros<-function(data){

  sum.tot<-apply(data$Data, 1, sum)
  
  data.new<-data$Data[sum.tot>0,]
  annotation.new<-data$Annotation[sum.tot>0,]
  
  data.struct<-data
  data.struct$Data<-data.new
  data.struct$Annotation<-annotation.new
  
  return(data.struct)

}


getSamples<-function(data, samples){

  newdata<-data
  newdata$Data<-data$Data[,samples]
  
  return(newdata)

}
